// ===========================
// CLASS Rocket				
// ===========================

var Rocket = function(settings) {
	settings.y = document.getElementById(configuration.canvasID).height;
	BaseObject.call(this, settings);
	
	this.lastPosition = {x: settings.x, y: settings.y};
	this.angle = settings.angle == "undefined" ? 90 : settings.angle;
	
	this.audioStart = new Audio("sounds/rocket-start.mp3");
	this.audioExplosion = new Audio("sounds/rocket-explosion.mp3");
	
	this.exploded = false;
	this.shape = new window[settings.explosionShape]()

	this.colors = [];
	this.substances = [];
	this.emitters = [];
	
	for(var i = 0; i < settings.substances.length; i++) {
		this.addSubstance(new window[settings.substances[i].type]());
	}
	
	if(this.colors.length > 0)
		this.color = mixColors(this.colors);
	else
		this.color = colors.dodger;
};

Rocket.prototype = Object.create(BaseObject.prototype);
Rocket.prototype.constructor = Rocket;

Rocket.prototype.isEmpty = function() {
	return this.emitters.length == 0 ? true : false;
}

Rocket.prototype.removeEmitter = function(emitter) {
	var index = this.emitters.indexOf(emitter);
	
	if(index != -1) {
		console.log("ROCKET: Emitter removed");
		this.emitters.splice(index, 1);
	}	
}

Rocket.prototype.updateEmitters = function(timeElapsed) {
	for(var i = 0; i < this.emitters.length; i++) {
		this.emitters[i].position.x = this.position.x;
		this.emitters[i].position.y = this.position.y;

		this.emitters[i].update(timeElapsed);
		
		if(!this.emitters[i].isAlive()) {
			this.emitters[i].remove = true;
		}
	}
}

Rocket.prototype.renderEmitters = function() {
	for(var i = 0; i < this.emitters.length; i++) {
		this.emitters[i].render();
	}
}

Rocket.prototype.removeDeadEmitters = function() {
	for(var i = 0; i < this.emitters.length; i++) {
		if(this.emitters[i].remove) { 
			console.log("ROCKET: Dead emitter removed");
			this.removeEmitter(this.emitters[i]);
		}
	}
}

Rocket.prototype.explode = function() {
	console.log("ROCKET["+this.id+"] Exploded");
	this.audioExplosion.play();
			
	this.exploded = true;
			
	this.emitters = [];
	
	this.shape.settings.color = this.color;
	
	this.emitters.push(new ParticleEmitter(this.position, this.shape.settings));
}

Rocket.prototype.update = function(timeElapsed) {
	this.lastPosition.x = this.position.x;
	this.lastPosition.y = this.position.y
		
	if(this.born) {
		this.born = false;
		this.audioStart.play();
	}

	if(!this.exploded) {
		this.speed *= this.friction;
		
		this.position.x += Math.cos(degreesToRadians(this.angle)) * this.speed + this.gravity.x;
		this.position.y += -(Math.sin(degreesToRadians(this.angle)) * this.speed) + this.gravity.y;
		
		this.age += timeElapsed;
		
		var ageRatio = this.age / this.lifespan;
		this.alpha = 1.0-ageRatio;
		
		if(!this.isAlive() || this.position.y < 0 || this.position.x < 0 || this.position.x > document.getElementById("board").width) {
			this.explode();
		}
	}
	
	this.updateEmitters(timeElapsed);
	this.removeDeadEmitters();
};

Rocket.prototype.render = function() {
	
	if(!this.exploded) {
		var canvas = document.getElementById(configuration.canvasID);
		c = canvas.getContext('2d');
		
		c.strokeStyle = "rgba("+ this.color.R+","+ this.color.G +","+ this.color.B +", "+this.alpha+")"; 
		
		c.beginPath();
			c.lineWidth = 3;
			c.moveTo(this.lastPosition.x,this.lastPosition.y);
			c.lineTo(this.position.x, this.position.y);
			c.stroke();
		c.closePath();
	}
	
	this.renderEmitters();
}

Rocket.prototype.addSubstance = function(substance) {
	this.substances.push(substance);

	if(substance.color != "undefined") this.colors.push(substance.color);
	if(substance.emitter != "undefined") this.emitters.push(new ParticleEmitter(this.position, substance.emitter));	
	
};
